from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseTOTPConfig")


@attr.s(auto_attribs=True)
class BaseTOTPConfig:
    """
    Attributes:
        enabled (Union[Unset, bool]):
        config_name (Union[Unset, str]): This name must be defined within the "totp" section of the SFTPGo configuration
            file. You will be unable to save a user/admin referencing a missing config_name
        secret (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must provide a
            payload and set the status to "Plain". The encryption key and additional data will be generated automatically.
            If you set the status to "Redacted" the existig secret will be preserved
    """

    enabled: Union[Unset, bool] = UNSET
    config_name: Union[Unset, str] = UNSET
    secret: Union[Unset, Secret] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        enabled = self.enabled
        config_name = self.config_name
        secret: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.secret, Unset):
            secret = self.secret.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if config_name is not UNSET:
            field_dict["config_name"] = config_name
        if secret is not UNSET:
            field_dict["secret"] = secret

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        enabled = d.pop("enabled", UNSET)

        config_name = d.pop("config_name", UNSET)

        _secret = d.pop("secret", UNSET)
        secret: Union[Unset, Secret]
        if isinstance(_secret, Unset):
            secret = UNSET
        else:
            secret = Secret.from_dict(_secret)

        base_totp_config = cls(
            enabled=enabled,
            config_name=config_name,
            secret=secret,
        )

        base_totp_config.additional_properties = d
        return base_totp_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import Client
from ...models.api_response import ApiResponse
from ...types import UNSET, Response


def _get_kwargs(
    *,
    client: Client,
    path: str,
) -> Dict[str, Any]:
    url = "{}/user/dirs".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["path"] = path

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "delete",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Any, List[ApiResponse]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = ApiResponse.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Any, List[ApiResponse]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    path: str,
) -> Response[Union[Any, List[ApiResponse]]]:
    """Delete a directory

     Delete a directory for the logged in user. Only empty directories can be deleted

    Args:
        path (str):

    Returns:
        Response[Union[Any, List[ApiResponse]]]
    """

    kwargs = _get_kwargs(
        client=client,
        path=path,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    path: str,
) -> Optional[Union[Any, List[ApiResponse]]]:
    """Delete a directory

     Delete a directory for the logged in user. Only empty directories can be deleted

    Args:
        path (str):

    Returns:
        Response[Union[Any, List[ApiResponse]]]
    """

    return sync_detailed(
        client=client,
        path=path,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    path: str,
) -> Response[Union[Any, List[ApiResponse]]]:
    """Delete a directory

     Delete a directory for the logged in user. Only empty directories can be deleted

    Args:
        path (str):

    Returns:
        Response[Union[Any, List[ApiResponse]]]
    """

    kwargs = _get_kwargs(
        client=client,
        path=path,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    path: str,
) -> Optional[Union[Any, List[ApiResponse]]]:
    """Delete a directory

     Delete a directory for the logged in user. Only empty directories can be deleted

    Args:
        path (str):

    Returns:
        Response[Union[Any, List[ApiResponse]]]
    """

    return (
        await asyncio_detailed(
            client=client,
            path=path,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.provider_event_action import ProviderEventAction
from ..models.provider_event_object_type import ProviderEventObjectType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ProviderEvent")


@attr.s(auto_attribs=True)
class ProviderEvent:
    """
    Attributes:
        id (Union[Unset, str]):
        timestamp (Union[Unset, int]): unix timestamp in nanoseconds
        action (Union[Unset, ProviderEventAction]):
        username (Union[Unset, str]):
        ip (Union[Unset, str]):
        object_type (Union[Unset, ProviderEventObjectType]):
        object_name (Union[Unset, str]):
        instance_id (Union[Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    timestamp: Union[Unset, int] = UNSET
    action: Union[Unset, ProviderEventAction] = UNSET
    username: Union[Unset, str] = UNSET
    ip: Union[Unset, str] = UNSET
    object_type: Union[Unset, ProviderEventObjectType] = UNSET
    object_name: Union[Unset, str] = UNSET
    instance_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        timestamp = self.timestamp
        action: Union[Unset, str] = UNSET
        if not isinstance(self.action, Unset):
            action = self.action.value

        username = self.username
        ip = self.ip
        object_type: Union[Unset, str] = UNSET
        if not isinstance(self.object_type, Unset):
            object_type = self.object_type.value

        object_name = self.object_name
        instance_id = self.instance_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if action is not UNSET:
            field_dict["action"] = action
        if username is not UNSET:
            field_dict["username"] = username
        if ip is not UNSET:
            field_dict["ip"] = ip
        if object_type is not UNSET:
            field_dict["object_type"] = object_type
        if object_name is not UNSET:
            field_dict["object_name"] = object_name
        if instance_id is not UNSET:
            field_dict["instance_id"] = instance_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        timestamp = d.pop("timestamp", UNSET)

        _action = d.pop("action", UNSET)
        action: Union[Unset, ProviderEventAction]
        if isinstance(_action, Unset):
            action = UNSET
        else:
            action = ProviderEventAction(_action)

        username = d.pop("username", UNSET)

        ip = d.pop("ip", UNSET)

        _object_type = d.pop("object_type", UNSET)
        object_type: Union[Unset, ProviderEventObjectType]
        if isinstance(_object_type, Unset):
            object_type = UNSET
        else:
            object_type = ProviderEventObjectType(_object_type)

        object_name = d.pop("object_name", UNSET)

        instance_id = d.pop("instance_id", UNSET)

        provider_event = cls(
            id=id,
            timestamp=timestamp,
            action=action,
            username=username,
            ip=ip,
            object_type=object_type,
            object_name=object_name,
            instance_id=instance_id,
        )

        provider_event.additional_properties = d
        return provider_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ProviderEventObjectType(str, Enum):
    USER = "user"
    ADMIN = "admin"
    API_KEY = "api_key"
    SHARE = "share"

    def __str__(self) -> str:
        return str(self.value)



from enum import IntEnum


class FsProviders(IntEnum):
    VALUE_0 = 0
    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_3 = 3
    VALUE_4 = 4
    VALUE_5 = 5

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.api_response import ApiResponse
from ...types import Response


def _get_kwargs(
    username: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/users/{username}/forgot-password".format(
        client.base_url, username=username
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, ApiResponse]]:
    if response.status_code == 200:
        response_200 = ApiResponse.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, ApiResponse]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    username: str,
    *,
    client: Client,
) -> Response[Union[Any, ApiResponse]]:
    """Send a password reset code by email

     You must configure an SMTP server, the account must have a valid email address and must not have the
    \"reset-password-disabled\" restriction, in which case SFTPGo will send a code via email to reset
    the password. If the specified user does not exist, the request will be silently ignored (a success
    response will be returned) to avoid disclosing existing users

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    kwargs = _get_kwargs(
        username=username,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    username: str,
    *,
    client: Client,
) -> Optional[Union[Any, ApiResponse]]:
    """Send a password reset code by email

     You must configure an SMTP server, the account must have a valid email address and must not have the
    \"reset-password-disabled\" restriction, in which case SFTPGo will send a code via email to reset
    the password. If the specified user does not exist, the request will be silently ignored (a success
    response will be returned) to avoid disclosing existing users

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    return sync_detailed(
        username=username,
        client=client,
    ).parsed


async def asyncio_detailed(
    username: str,
    *,
    client: Client,
) -> Response[Union[Any, ApiResponse]]:
    """Send a password reset code by email

     You must configure an SMTP server, the account must have a valid email address and must not have the
    \"reset-password-disabled\" restriction, in which case SFTPGo will send a code via email to reset
    the password. If the specified user does not exist, the request will be silently ignored (a success
    response will be returned) to avoid disclosing existing users

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    kwargs = _get_kwargs(
        username=username,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    username: str,
    *,
    client: Client,
) -> Optional[Union[Any, ApiResponse]]:
    """Send a password reset code by email

     You must configure an SMTP server, the account must have a valid email address and must not have the
    \"reset-password-disabled\" restriction, in which case SFTPGo will send a code via email to reset
    the password. If the specified user does not exist, the request will be silently ignored (a success
    response will be returned) to avoid disclosing existing users

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    return (
        await asyncio_detailed(
            username=username,
            client=client,
        )
    ).parsed



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.api_response import ApiResponse
from ...types import Response


def _get_kwargs(
    username: str,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/admins/{username}/forgot-password".format(
        client.base_url, username=username
    )

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, ApiResponse]]:
    if response.status_code == 200:
        response_200 = ApiResponse.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, ApiResponse]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    username: str,
    *,
    client: Client,
) -> Response[Union[Any, ApiResponse]]:
    """Send a password reset code by email

     You must set up an SMTP server and the account must have a valid email address, in which case SFTPGo
    will send a code via email to reset the password. If the specified admin does not exist, the request
    will be silently ignored (a success response will be returned) to avoid disclosing existing admins

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    kwargs = _get_kwargs(
        username=username,
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    username: str,
    *,
    client: Client,
) -> Optional[Union[Any, ApiResponse]]:
    """Send a password reset code by email

     You must set up an SMTP server and the account must have a valid email address, in which case SFTPGo
    will send a code via email to reset the password. If the specified admin does not exist, the request
    will be silently ignored (a success response will be returned) to avoid disclosing existing admins

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    return sync_detailed(
        username=username,
        client=client,
    ).parsed


async def asyncio_detailed(
    username: str,
    *,
    client: Client,
) -> Response[Union[Any, ApiResponse]]:
    """Send a password reset code by email

     You must set up an SMTP server and the account must have a valid email address, in which case SFTPGo
    will send a code via email to reset the password. If the specified admin does not exist, the request
    will be silently ignored (a success response will be returned) to avoid disclosing existing admins

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    kwargs = _get_kwargs(
        username=username,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    username: str,
    *,
    client: Client,
) -> Optional[Union[Any, ApiResponse]]:
    """Send a password reset code by email

     You must set up an SMTP server and the account must have a valid email address, in which case SFTPGo
    will send a code via email to reset the password. If the specified admin does not exist, the request
    will be silently ignored (a success response will be returned) to avoid disclosing existing admins

    Args:
        username (str):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    return (
        await asyncio_detailed(
            username=username,
            client=client,
        )
    ).parsed



from enum import Enum


class WebClientOptions(str, Enum):
    PUBLICKEY_CHANGE_DISABLED = "publickey-change-disabled"
    WRITE_DISABLED = "write-disabled"
    MFA_DISABLED = "mfa-disabled"
    PASSWORD_CHANGE_DISABLED = "password-change-disabled"
    API_KEY_AUTH_CHANGE_DISABLED = "api-key-auth-change-disabled"
    INFO_CHANGE_DISABLED = "info-change-disabled"
    SHARES_DISABLED = "shares-disabled"
    PASSWORD_RESET_DISABLED = "password-reset-disabled"
    SHARES_WITHOUT_PASSWORD_DISABLED = "shares-without-password-disabled"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="ValidateAdminTotpSecretJsonBody")


@attr.s(auto_attribs=True)
class ValidateAdminTotpSecretJsonBody:
    """
    Attributes:
        config_name (Union[Unset, str]): name of the configuration to use to validate the passcode
        passcode (Union[Unset, str]): passcode to validate
        secret (Union[Unset, str]): secret to use to validate the passcode
    """

    config_name: Union[Unset, str] = UNSET
    passcode: Union[Unset, str] = UNSET
    secret: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        config_name = self.config_name
        passcode = self.passcode
        secret = self.secret

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if config_name is not UNSET:
            field_dict["config_name"] = config_name
        if passcode is not UNSET:
            field_dict["passcode"] = passcode
        if secret is not UNSET:
            field_dict["secret"] = secret

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        config_name = d.pop("config_name", UNSET)

        passcode = d.pop("passcode", UNSET)

        secret = d.pop("secret", UNSET)

        validate_admin_totp_secret_json_body = cls(
            config_name=config_name,
            passcode=passcode,
            secret=secret,
        )

        validate_admin_totp_secret_json_body.additional_properties = d
        return validate_admin_totp_secret_json_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.event_protocols import EventProtocols
from ..models.fs_event_action import FsEventAction
from ..models.fs_event_status import FsEventStatus
from ..models.fs_providers import FsProviders
from ..types import UNSET, Unset

T = TypeVar("T", bound="FsEvent")


@attr.s(auto_attribs=True)
class FsEvent:
    """
    Attributes:
        id (Union[Unset, str]):
        timestamp (Union[Unset, int]): unix timestamp in nanoseconds
        action (Union[Unset, FsEventAction]):
        username (Union[Unset, str]):
        fs_path (Union[Unset, str]):
        fs_target_path (Union[Unset, str]):
        virtual_path (Union[Unset, str]):
        virtual_target_path (Union[Unset, str]):
        ssh_cmd (Union[Unset, str]):
        file_size (Union[Unset, int]):
        status (Union[Unset, FsEventStatus]): Event status:
              * `1` - no error
              * `2` - generic error
              * `3` - quota exceeded error
        protocol (Union[Unset, EventProtocols]): Protocols:
              * `SSH` - SSH commands
              * `SFTP` - SFTP protocol
              * `FTP` - plain FTP and FTPES/FTPS
              * `DAV` - WebDAV
              * `HTTP` - WebClient/REST API
              * `HTTPShare` - the event is generated in a public share
              * `DataRetention` - the event is generated by a data retention check
              * `OIDC` - OpenID Connect
        ip (Union[Unset, str]):
        session_id (Union[Unset, str]):
        fs_provider (Union[Unset, FsProviders]): Filesystem providers:
              * `0` - Local filesystem
              * `1` - S3 Compatible Object Storage
              * `2` - Google Cloud Storage
              * `3` - Azure Blob Storage
              * `4` - Local filesystem encrypted
              * `5` - SFTP
        bucket (Union[Unset, str]):
        endpoint (Union[Unset, str]):
        open_flags (Union[Unset, str]):
        instance_id (Union[Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    timestamp: Union[Unset, int] = UNSET
    action: Union[Unset, FsEventAction] = UNSET
    username: Union[Unset, str] = UNSET
    fs_path: Union[Unset, str] = UNSET
    fs_target_path: Union[Unset, str] = UNSET
    virtual_path: Union[Unset, str] = UNSET
    virtual_target_path: Union[Unset, str] = UNSET
    ssh_cmd: Union[Unset, str] = UNSET
    file_size: Union[Unset, int] = UNSET
    status: Union[Unset, FsEventStatus] = UNSET
    protocol: Union[Unset, EventProtocols] = UNSET
    ip: Union[Unset, str] = UNSET
    session_id: Union[Unset, str] = UNSET
    fs_provider: Union[Unset, FsProviders] = UNSET
    bucket: Union[Unset, str] = UNSET
    endpoint: Union[Unset, str] = UNSET
    open_flags: Union[Unset, str] = UNSET
    instance_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        timestamp = self.timestamp
        action: Union[Unset, str] = UNSET
        if not isinstance(self.action, Unset):
            action = self.action.value

        username = self.username
        fs_path = self.fs_path
        fs_target_path = self.fs_target_path
        virtual_path = self.virtual_path
        virtual_target_path = self.virtual_target_path
        ssh_cmd = self.ssh_cmd
        file_size = self.file_size
        status: Union[Unset, int] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        protocol: Union[Unset, str] = UNSET
        if not isinstance(self.protocol, Unset):
            protocol = self.protocol.value

        ip = self.ip
        session_id = self.session_id
        fs_provider: Union[Unset, int] = UNSET
        if not isinstance(self.fs_provider, Unset):
            fs_provider = self.fs_provider.value

        bucket = self.bucket
        endpoint = self.endpoint
        open_flags = self.open_flags
        instance_id = self.instance_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if action is not UNSET:
            field_dict["action"] = action
        if username is not UNSET:
            field_dict["username"] = username
        if fs_path is not UNSET:
            field_dict["fs_path"] = fs_path
        if fs_target_path is not UNSET:
            field_dict["fs_target_path"] = fs_target_path
        if virtual_path is not UNSET:
            field_dict["virtual_path"] = virtual_path
        if virtual_target_path is not UNSET:
            field_dict["virtual_target_path"] = virtual_target_path
        if ssh_cmd is not UNSET:
            field_dict["ssh_cmd"] = ssh_cmd
        if file_size is not UNSET:
            field_dict["file_size"] = file_size
        if status is not UNSET:
            field_dict["status"] = status
        if protocol is not UNSET:
            field_dict["protocol"] = protocol
        if ip is not UNSET:
            field_dict["ip"] = ip
        if session_id is not UNSET:
            field_dict["session_id"] = session_id
        if fs_provider is not UNSET:
            field_dict["fs_provider"] = fs_provider
        if bucket is not UNSET:
            field_dict["bucket"] = bucket
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if open_flags is not UNSET:
            field_dict["open_flags"] = open_flags
        if instance_id is not UNSET:
            field_dict["instance_id"] = instance_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        timestamp = d.pop("timestamp", UNSET)

        _action = d.pop("action", UNSET)
        action: Union[Unset, FsEventAction]
        if isinstance(_action, Unset):
            action = UNSET
        else:
            action = FsEventAction(_action)

        username = d.pop("username", UNSET)

        fs_path = d.pop("fs_path", UNSET)

        fs_target_path = d.pop("fs_target_path", UNSET)

        virtual_path = d.pop("virtual_path", UNSET)

        virtual_target_path = d.pop("virtual_target_path", UNSET)

        ssh_cmd = d.pop("ssh_cmd", UNSET)

        file_size = d.pop("file_size", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, FsEventStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = FsEventStatus(_status)

        _protocol = d.pop("protocol", UNSET)
        protocol: Union[Unset, EventProtocols]
        if isinstance(_protocol, Unset):
            protocol = UNSET
        else:
            protocol = EventProtocols(_protocol)

        ip = d.pop("ip", UNSET)

        session_id = d.pop("session_id", UNSET)

        _fs_provider = d.pop("fs_provider", UNSET)
        fs_provider: Union[Unset, FsProviders]
        if isinstance(_fs_provider, Unset):
            fs_provider = UNSET
        else:
            fs_provider = FsProviders(_fs_provider)

        bucket = d.pop("bucket", UNSET)

        endpoint = d.pop("endpoint", UNSET)

        open_flags = d.pop("open_flags", UNSET)

        instance_id = d.pop("instance_id", UNSET)

        fs_event = cls(
            id=id,
            timestamp=timestamp,
            action=action,
            username=username,
            fs_path=fs_path,
            fs_target_path=fs_target_path,
            virtual_path=virtual_path,
            virtual_target_path=virtual_target_path,
            ssh_cmd=ssh_cmd,
            file_size=file_size,
            status=status,
            protocol=protocol,
            ip=ip,
            session_id=session_id,
            fs_provider=fs_provider,
            bucket=bucket,
            endpoint=endpoint,
            open_flags=open_flags,
            instance_id=instance_id,
        )

        fs_event.additional_properties = d
        return fs_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.patterns_filter_deny_policy import PatternsFilterDenyPolicy
from ..types import UNSET, Unset

T = TypeVar("T", bound="PatternsFilter")


@attr.s(auto_attribs=True)
class PatternsFilter:
    """
    Attributes:
        path (Union[Unset, str]): exposed virtual path, if no other specific filter is defined, the filter applies for
            sub directories too. For example if filters are defined for the paths "/" and "/sub" then the filters for "/"
            are applied for any file outside the "/sub" directory
        allowed_patterns (Union[Unset, List[str]]): list of, case insensitive, allowed shell like patterns. Example:
            ['*.jpg', 'a*b?.png'].
        denied_patterns (Union[Unset, List[str]]): list of, case insensitive, denied shell like patterns. Denied
            patterns are evaluated before the allowed ones Example: ['*.zip'].
        deny_policy (Union[Unset, PatternsFilterDenyPolicy]): Deny policies
              * `0` - default policy. Denied files/directories matching the filters are visible in directory listing but
            cannot be uploaded/downloaded/overwritten/renamed
              * `1` - deny policy hide. This policy applies the same restrictions as the default one and denied
            files/directories matching the filters will also be hidden in directory listing. This mode may cause performance
            issues for large directories
    """

    path: Union[Unset, str] = UNSET
    allowed_patterns: Union[Unset, List[str]] = UNSET
    denied_patterns: Union[Unset, List[str]] = UNSET
    deny_policy: Union[Unset, PatternsFilterDenyPolicy] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        path = self.path
        allowed_patterns: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_patterns, Unset):
            allowed_patterns = self.allowed_patterns

        denied_patterns: Union[Unset, List[str]] = UNSET
        if not isinstance(self.denied_patterns, Unset):
            denied_patterns = self.denied_patterns

        deny_policy: Union[Unset, int] = UNSET
        if not isinstance(self.deny_policy, Unset):
            deny_policy = self.deny_policy.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if path is not UNSET:
            field_dict["path"] = path
        if allowed_patterns is not UNSET:
            field_dict["allowed_patterns"] = allowed_patterns
        if denied_patterns is not UNSET:
            field_dict["denied_patterns"] = denied_patterns
        if deny_policy is not UNSET:
            field_dict["deny_policy"] = deny_policy

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        path = d.pop("path", UNSET)

        allowed_patterns = cast(List[str], d.pop("allowed_patterns", UNSET))

        denied_patterns = cast(List[str], d.pop("denied_patterns", UNSET))

        _deny_policy = d.pop("deny_policy", UNSET)
        deny_policy: Union[Unset, PatternsFilterDenyPolicy]
        if isinstance(_deny_policy, Unset):
            deny_policy = UNSET
        else:
            deny_policy = PatternsFilterDenyPolicy(_deny_policy)

        patterns_filter = cls(
            path=path,
            allowed_patterns=allowed_patterns,
            denied_patterns=denied_patterns,
            deny_policy=deny_policy,
        )

        patterns_filter.additional_properties = d
        return patterns_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BandwidthLimit")


@attr.s(auto_attribs=True)
class BandwidthLimit:
    """
    Attributes:
        sources (Union[Unset, List[str]]): Source networks in CIDR notation as defined in RFC 4632 and RFC 4291 for
            example `192.0.2.0/24` or `2001:db8::/32`. The limit applies if the defined networks contain the client IP
        upload_bandwidth (Union[Unset, int]): Maximum upload bandwidth as KB/s, 0 means unlimited
        download_bandwidth (Union[Unset, int]): Maximum download bandwidth as KB/s, 0 means unlimited
    """

    sources: Union[Unset, List[str]] = UNSET
    upload_bandwidth: Union[Unset, int] = UNSET
    download_bandwidth: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        sources: Union[Unset, List[str]] = UNSET
        if not isinstance(self.sources, Unset):
            sources = self.sources

        upload_bandwidth = self.upload_bandwidth
        download_bandwidth = self.download_bandwidth

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if sources is not UNSET:
            field_dict["sources"] = sources
        if upload_bandwidth is not UNSET:
            field_dict["upload_bandwidth"] = upload_bandwidth
        if download_bandwidth is not UNSET:
            field_dict["download_bandwidth"] = download_bandwidth

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        sources = cast(List[str], d.pop("sources", UNSET))

        upload_bandwidth = d.pop("upload_bandwidth", UNSET)

        download_bandwidth = d.pop("download_bandwidth", UNSET)

        bandwidth_limit = cls(
            sources=sources,
            upload_bandwidth=upload_bandwidth,
            download_bandwidth=download_bandwidth,
        )

        bandwidth_limit.additional_properties = d
        return bandwidth_limit

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Union

import httpx

from ...client import Client
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    path: str,
    inline: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/user/files".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["path"] = path

    params["inline"] = inline

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    *,
    client: Client,
    path: str,
    inline: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """Download a single file

     Returns the file contents as response body

    Args:
        path (str):
        inline (Union[Unset, None, str]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        path=path,
        inline=inline,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    *,
    client: Client,
    path: str,
    inline: Union[Unset, None, str] = UNSET,
) -> Response[Any]:
    """Download a single file

     Returns the file contents as response body

    Args:
        path (str):
        inline (Union[Unset, None, str]):

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        client=client,
        path=path,
        inline=inline,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.totph_mac_algo import TOTPHMacAlgo
from ..types import UNSET, Unset

T = TypeVar("T", bound="TOTPConfig")


@attr.s(auto_attribs=True)
class TOTPConfig:
    """
    Attributes:
        name (Union[Unset, str]):
        issuer (Union[Unset, str]):
        algo (Union[Unset, TOTPHMacAlgo]): Supported HMAC algorithms for Time-based one time passwords
    """

    name: Union[Unset, str] = UNSET
    issuer: Union[Unset, str] = UNSET
    algo: Union[Unset, TOTPHMacAlgo] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        issuer = self.issuer
        algo: Union[Unset, str] = UNSET
        if not isinstance(self.algo, Unset):
            algo = self.algo.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if issuer is not UNSET:
            field_dict["issuer"] = issuer
        if algo is not UNSET:
            field_dict["algo"] = algo

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        issuer = d.pop("issuer", UNSET)

        _algo = d.pop("algo", UNSET)
        algo: Union[Unset, TOTPHMacAlgo]
        if isinstance(_algo, Unset):
            algo = UNSET
        else:
            algo = TOTPHMacAlgo(_algo)

        totp_config = cls(
            name=name,
            issuer=issuer,
            algo=algo,
        )

        totp_config.additional_properties = d
        return totp_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.folder_retention import FolderRetention
from ..models.retention_check_notification import RetentionCheckNotification
from ..types import UNSET, Unset

T = TypeVar("T", bound="RetentionCheck")


@attr.s(auto_attribs=True)
class RetentionCheck:
    """
    Attributes:
        username (Union[Unset, str]): username to which the retention check refers
        folders (Union[Unset, List[FolderRetention]]):
        start_time (Union[Unset, int]): check start time as unix timestamp in milliseconds
        notifications (Union[Unset, List[RetentionCheckNotification]]):
        email (Union[Unset, str]): if the notification method is set to "Email", this is the e-mail address that
            receives the retention check report. This field is automatically set to the email address associated with the
            administrator starting the check
    """

    username: Union[Unset, str] = UNSET
    folders: Union[Unset, List[FolderRetention]] = UNSET
    start_time: Union[Unset, int] = UNSET
    notifications: Union[Unset, List[RetentionCheckNotification]] = UNSET
    email: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        username = self.username
        folders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.folders, Unset):
            folders = []
            for folders_item_data in self.folders:
                folders_item = folders_item_data.to_dict()

                folders.append(folders_item)

        start_time = self.start_time
        notifications: Union[Unset, List[str]] = UNSET
        if not isinstance(self.notifications, Unset):
            notifications = []
            for notifications_item_data in self.notifications:
                notifications_item = notifications_item_data.value

                notifications.append(notifications_item)

        email = self.email

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if username is not UNSET:
            field_dict["username"] = username
        if folders is not UNSET:
            field_dict["folders"] = folders
        if start_time is not UNSET:
            field_dict["start_time"] = start_time
        if notifications is not UNSET:
            field_dict["notifications"] = notifications
        if email is not UNSET:
            field_dict["email"] = email

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        username = d.pop("username", UNSET)

        folders = []
        _folders = d.pop("folders", UNSET)
        for folders_item_data in _folders or []:
            folders_item = FolderRetention.from_dict(folders_item_data)

            folders.append(folders_item)

        start_time = d.pop("start_time", UNSET)

        notifications = []
        _notifications = d.pop("notifications", UNSET)
        for notifications_item_data in _notifications or []:
            notifications_item = RetentionCheckNotification(notifications_item_data)

            notifications.append(notifications_item)

        email = d.pop("email", UNSET)

        retention_check = cls(
            username=username,
            folders=folders,
            start_time=start_time,
            notifications=notifications,
            email=email,
        )

        retention_check.additional_properties = d
        return retention_check

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="SFTPFsConfig")


@attr.s(auto_attribs=True)
class SFTPFsConfig:
    """
    Attributes:
        endpoint (Union[Unset, str]): remote SFTP endpoint as host:port
        username (Union[Unset, str]): you can specify a password or private key or both. In the latter case the private
            key will be tried first.
        password (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must provide
            a payload and set the status to "Plain". The encryption key and additional data will be generated automatically.
            If you set the status to "Redacted" the existig secret will be preserved
        private_key (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must
            provide a payload and set the status to "Plain". The encryption key and additional data will be generated
            automatically. If you set the status to "Redacted" the existig secret will be preserved
        fingerprints (Union[Unset, List[str]]): SHA256 fingerprints to use for host key verification. If you don't
            provide any fingerprint the remote host key will not be verified, this is a security risk
        prefix (Union[Unset, str]): Specifying a prefix you can restrict all operations to a given path within the
            remote SFTP server.
        disable_concurrent_reads (Union[Unset, bool]): Concurrent reads are safe to use and disabling them will degrade
            performance. Some servers automatically delete files once they are downloaded. Using concurrent reads is
            problematic with such servers.
        buffer_size (Union[Unset, int]): The size of the buffer (in MB) to use for transfers. By enabling buffering, the
            reads and writes, from/to the remote SFTP server, are split in multiple concurrent requests and this allows data
            to be transferred at a faster rate, over high latency networks, by overlapping round-trip times. With buffering
            enabled, resuming uploads is not supported and a file cannot be opened for both reading and writing at the same
            time. 0 means disabled. Example: 2.
    """

    endpoint: Union[Unset, str] = UNSET
    username: Union[Unset, str] = UNSET
    password: Union[Unset, Secret] = UNSET
    private_key: Union[Unset, Secret] = UNSET
    fingerprints: Union[Unset, List[str]] = UNSET
    prefix: Union[Unset, str] = UNSET
    disable_concurrent_reads: Union[Unset, bool] = UNSET
    buffer_size: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        endpoint = self.endpoint
        username = self.username
        password: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.password, Unset):
            password = self.password.to_dict()

        private_key: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.private_key, Unset):
            private_key = self.private_key.to_dict()

        fingerprints: Union[Unset, List[str]] = UNSET
        if not isinstance(self.fingerprints, Unset):
            fingerprints = self.fingerprints

        prefix = self.prefix
        disable_concurrent_reads = self.disable_concurrent_reads
        buffer_size = self.buffer_size

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if username is not UNSET:
            field_dict["username"] = username
        if password is not UNSET:
            field_dict["password"] = password
        if private_key is not UNSET:
            field_dict["private_key"] = private_key
        if fingerprints is not UNSET:
            field_dict["fingerprints"] = fingerprints
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if disable_concurrent_reads is not UNSET:
            field_dict["disable_concurrent_reads"] = disable_concurrent_reads
        if buffer_size is not UNSET:
            field_dict["buffer_size"] = buffer_size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        endpoint = d.pop("endpoint", UNSET)

        username = d.pop("username", UNSET)

        _password = d.pop("password", UNSET)
        password: Union[Unset, Secret]
        if isinstance(_password, Unset):
            password = UNSET
        else:
            password = Secret.from_dict(_password)

        _private_key = d.pop("private_key", UNSET)
        private_key: Union[Unset, Secret]
        if isinstance(_private_key, Unset):
            private_key = UNSET
        else:
            private_key = Secret.from_dict(_private_key)

        fingerprints = cast(List[str], d.pop("fingerprints", UNSET))

        prefix = d.pop("prefix", UNSET)

        disable_concurrent_reads = d.pop("disable_concurrent_reads", UNSET)

        buffer_size = d.pop("buffer_size", UNSET)

        sftp_fs_config = cls(
            endpoint=endpoint,
            username=username,
            password=password,
            private_key=private_key,
            fingerprints=fingerprints,
            prefix=prefix,
            disable_concurrent_reads=disable_concurrent_reads,
            buffer_size=buffer_size,
        )

        sftp_fs_config.additional_properties = d
        return sftp_fs_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="S3Config")


@attr.s(auto_attribs=True)
class S3Config:
    """S3 Compatible Object Storage configuration details

    Attributes:
        bucket (Union[Unset, str]):
        region (Union[Unset, str]):
        access_key (Union[Unset, str]):
        access_secret (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must
            provide a payload and set the status to "Plain". The encryption key and additional data will be generated
            automatically. If you set the status to "Redacted" the existig secret will be preserved
        role_arn (Union[Unset, str]): Optional IAM Role ARN to assume
        endpoint (Union[Unset, str]): optional endpoint
        storage_class (Union[Unset, str]):
        acl (Union[Unset, str]): The canned ACL to apply to uploaded objects. Leave empty to use the default ACL. For
            more information and available ACLs, see here: https://docs.aws.amazon.com/AmazonS3/latest/userguide/acl-
            overview.html#canned-acl
        upload_part_size (Union[Unset, int]): the buffer size (in MB) to use for multipart uploads. The minimum allowed
            part size is 5MB, and if this value is set to zero, the default value (5MB) for the AWS SDK will be used. The
            minimum allowed value is 5.
        upload_concurrency (Union[Unset, int]): the number of parts to upload in parallel. If this value is set to zero,
            the default value (5) will be used
        upload_part_max_time (Union[Unset, int]): the maximum time allowed, in seconds, to upload a single chunk (the
            chunk size is defined via "upload_part_size"). 0 means no timeout
        download_part_size (Union[Unset, int]): the buffer size (in MB) to use for multipart downloads. The minimum
            allowed part size is 5MB, and if this value is set to zero, the default value (5MB) for the AWS SDK will be
            used. The minimum allowed value is 5. Ignored for partial downloads
        download_concurrency (Union[Unset, int]): the number of parts to download in parallel. If this value is set to
            zero, the default value (5) will be used. Ignored for partial downloads
        download_part_max_time (Union[Unset, int]): the maximum time allowed, in seconds, to download a single chunk
            (the chunk size is defined via "download_part_size"). 0 means no timeout. Ignored for partial downloads.
        force_path_style (Union[Unset, bool]): Set this to "true" to force the request to use path-style addressing,
            i.e., "http://s3.amazonaws.com/BUCKET/KEY". By default, the S3 client will use virtual hosted bucket addressing
            when possible ("http://BUCKET.s3.amazonaws.com/KEY")
        key_prefix (Union[Unset, str]): key_prefix is similar to a chroot directory for a local filesystem. If specified
            the user will only see contents that starts with this prefix and so you can restrict access to a specific
            virtual folder. The prefix, if not empty, must not start with "/" and must end with "/". If empty the whole
            bucket contents will be available Example: folder/subfolder/.
    """

    bucket: Union[Unset, str] = UNSET
    region: Union[Unset, str] = UNSET
    access_key: Union[Unset, str] = UNSET
    access_secret: Union[Unset, Secret] = UNSET
    role_arn: Union[Unset, str] = UNSET
    endpoint: Union[Unset, str] = UNSET
    storage_class: Union[Unset, str] = UNSET
    acl: Union[Unset, str] = UNSET
    upload_part_size: Union[Unset, int] = UNSET
    upload_concurrency: Union[Unset, int] = UNSET
    upload_part_max_time: Union[Unset, int] = UNSET
    download_part_size: Union[Unset, int] = UNSET
    download_concurrency: Union[Unset, int] = UNSET
    download_part_max_time: Union[Unset, int] = UNSET
    force_path_style: Union[Unset, bool] = UNSET
    key_prefix: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bucket = self.bucket
        region = self.region
        access_key = self.access_key
        access_secret: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.access_secret, Unset):
            access_secret = self.access_secret.to_dict()

        role_arn = self.role_arn
        endpoint = self.endpoint
        storage_class = self.storage_class
        acl = self.acl
        upload_part_size = self.upload_part_size
        upload_concurrency = self.upload_concurrency
        upload_part_max_time = self.upload_part_max_time
        download_part_size = self.download_part_size
        download_concurrency = self.download_concurrency
        download_part_max_time = self.download_part_max_time
        force_path_style = self.force_path_style
        key_prefix = self.key_prefix

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if bucket is not UNSET:
            field_dict["bucket"] = bucket
        if region is not UNSET:
            field_dict["region"] = region
        if access_key is not UNSET:
            field_dict["access_key"] = access_key
        if access_secret is not UNSET:
            field_dict["access_secret"] = access_secret
        if role_arn is not UNSET:
            field_dict["role_arn"] = role_arn
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if storage_class is not UNSET:
            field_dict["storage_class"] = storage_class
        if acl is not UNSET:
            field_dict["acl"] = acl
        if upload_part_size is not UNSET:
            field_dict["upload_part_size"] = upload_part_size
        if upload_concurrency is not UNSET:
            field_dict["upload_concurrency"] = upload_concurrency
        if upload_part_max_time is not UNSET:
            field_dict["upload_part_max_time"] = upload_part_max_time
        if download_part_size is not UNSET:
            field_dict["download_part_size"] = download_part_size
        if download_concurrency is not UNSET:
            field_dict["download_concurrency"] = download_concurrency
        if download_part_max_time is not UNSET:
            field_dict["download_part_max_time"] = download_part_max_time
        if force_path_style is not UNSET:
            field_dict["force_path_style"] = force_path_style
        if key_prefix is not UNSET:
            field_dict["key_prefix"] = key_prefix

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bucket = d.pop("bucket", UNSET)

        region = d.pop("region", UNSET)

        access_key = d.pop("access_key", UNSET)

        _access_secret = d.pop("access_secret", UNSET)
        access_secret: Union[Unset, Secret]
        if isinstance(_access_secret, Unset):
            access_secret = UNSET
        else:
            access_secret = Secret.from_dict(_access_secret)

        role_arn = d.pop("role_arn", UNSET)

        endpoint = d.pop("endpoint", UNSET)

        storage_class = d.pop("storage_class", UNSET)

        acl = d.pop("acl", UNSET)

        upload_part_size = d.pop("upload_part_size", UNSET)

        upload_concurrency = d.pop("upload_concurrency", UNSET)

        upload_part_max_time = d.pop("upload_part_max_time", UNSET)

        download_part_size = d.pop("download_part_size", UNSET)

        download_concurrency = d.pop("download_concurrency", UNSET)

        download_part_max_time = d.pop("download_part_max_time", UNSET)

        force_path_style = d.pop("force_path_style", UNSET)

        key_prefix = d.pop("key_prefix", UNSET)

        s3_config = cls(
            bucket=bucket,
            region=region,
            access_key=access_key,
            access_secret=access_secret,
            role_arn=role_arn,
            endpoint=endpoint,
            storage_class=storage_class,
            acl=acl,
            upload_part_size=upload_part_size,
            upload_concurrency=upload_concurrency,
            upload_part_max_time=upload_part_max_time,
            download_part_size=download_part_size,
            download_concurrency=download_concurrency,
            download_part_max_time=download_part_max_time,
            force_path_style=force_path_style,
            key_prefix=key_prefix,
        )

        s3_config.additional_properties = d
        return s3_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import AuthenticatedClient
from ...models.recovery_code import RecoveryCode
from ...types import Response


def _get_kwargs(
    *,
    client: AuthenticatedClient,
) -> Dict[str, Any]:
    url = "{}/admin/2fa/recoverycodes".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Any, List[RecoveryCode]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = RecoveryCode.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Any, List[RecoveryCode]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
) -> Response[Union[Any, List[RecoveryCode]]]:
    """Get recovery codes

     Returns the recovery codes for the logged in admin. Recovery codes can be used if the admin loses
    access to their second factor auth device. Recovery codes are returned unencrypted

    Returns:
        Response[Union[Any, List[RecoveryCode]]]
    """

    kwargs = _get_kwargs(
        client=client,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
) -> Optional[Union[Any, List[RecoveryCode]]]:
    """Get recovery codes

     Returns the recovery codes for the logged in admin. Recovery codes can be used if the admin loses
    access to their second factor auth device. Recovery codes are returned unencrypted

    Returns:
        Response[Union[Any, List[RecoveryCode]]]
    """

    return sync_detailed(
        client=client,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
) -> Response[Union[Any, List[RecoveryCode]]]:
    """Get recovery codes

     Returns the recovery codes for the logged in admin. Recovery codes can be used if the admin loses
    access to their second factor auth device. Recovery codes are returned unencrypted

    Returns:
        Response[Union[Any, List[RecoveryCode]]]
    """

    kwargs = _get_kwargs(
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
) -> Optional[Union[Any, List[RecoveryCode]]]:
    """Get recovery codes

     Returns the recovery codes for the logged in admin. Recovery codes can be used if the admin loses
    access to their second factor auth device. Recovery codes are returned unencrypted

    Returns:
        Response[Union[Any, List[RecoveryCode]]]
    """

    return (
        await asyncio_detailed(
            client=client,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.share_scope import ShareScope
from ..types import UNSET, Unset

T = TypeVar("T", bound="Share")


@attr.s(auto_attribs=True)
class Share:
    """
    Attributes:
        id (Union[Unset, str]): auto-generated unique share identifier
        name (Union[Unset, str]):
        description (Union[Unset, str]): optional description
        scope (Union[Unset, ShareScope]): Options:
              * `1` - read scope
              * `2` - write scope
        paths (Union[Unset, List[str]]): paths to files or directories, for share scope write this array must contain
            exactly one directory. Paths will not be validated on save so you can also create them after creating the share
            Example: ['/dir1', '/dir2/file.txt', '/dir3/subdir'].
        username (Union[Unset, str]):
        created_at (Union[Unset, int]): creation time as unix timestamp in milliseconds
        updated_at (Union[Unset, int]): last update time as unix timestamp in milliseconds
        last_use_at (Union[Unset, int]): last use time as unix timestamp in milliseconds
        expires_at (Union[Unset, int]): optional share expiration, as unix timestamp in milliseconds. 0 means no
            expiration
        password (Union[Unset, str]): optional password to protect the share. The special value "[**redacted**]" means
            that a password has been set, you can use this value if you want to preserve the current password when you
            update a share
        max_tokens (Union[Unset, int]): maximum allowed access tokens. 0 means no limit
        used_tokens (Union[Unset, int]):
        allow_from (Union[Unset, List[str]]): Limit the share availability to these IP/Mask. IP/Mask must be in CIDR
            notation as defined in RFC 4632 and RFC 4291, for example "192.0.2.0/24" or "2001:db8::/32". An empty list means
            no restrictions Example: ['192.0.2.0/24', '2001:db8::/32'].
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    scope: Union[Unset, ShareScope] = UNSET
    paths: Union[Unset, List[str]] = UNSET
    username: Union[Unset, str] = UNSET
    created_at: Union[Unset, int] = UNSET
    updated_at: Union[Unset, int] = UNSET
    last_use_at: Union[Unset, int] = UNSET
    expires_at: Union[Unset, int] = UNSET
    password: Union[Unset, str] = UNSET
    max_tokens: Union[Unset, int] = UNSET
    used_tokens: Union[Unset, int] = UNSET
    allow_from: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        description = self.description
        scope: Union[Unset, int] = UNSET
        if not isinstance(self.scope, Unset):
            scope = self.scope.value

        paths: Union[Unset, List[str]] = UNSET
        if not isinstance(self.paths, Unset):
            paths = self.paths

        username = self.username
        created_at = self.created_at
        updated_at = self.updated_at
        last_use_at = self.last_use_at
        expires_at = self.expires_at
        password = self.password
        max_tokens = self.max_tokens
        used_tokens = self.used_tokens
        allow_from: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allow_from, Unset):
            allow_from = self.allow_from

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if description is not UNSET:
            field_dict["description"] = description
        if scope is not UNSET:
            field_dict["scope"] = scope
        if paths is not UNSET:
            field_dict["paths"] = paths
        if username is not UNSET:
            field_dict["username"] = username
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if last_use_at is not UNSET:
            field_dict["last_use_at"] = last_use_at
        if expires_at is not UNSET:
            field_dict["expires_at"] = expires_at
        if password is not UNSET:
            field_dict["password"] = password
        if max_tokens is not UNSET:
            field_dict["max_tokens"] = max_tokens
        if used_tokens is not UNSET:
            field_dict["used_tokens"] = used_tokens
        if allow_from is not UNSET:
            field_dict["allow_from"] = allow_from

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        description = d.pop("description", UNSET)

        _scope = d.pop("scope", UNSET)
        scope: Union[Unset, ShareScope]
        if isinstance(_scope, Unset):
            scope = UNSET
        else:
            scope = ShareScope(_scope)

        paths = cast(List[str], d.pop("paths", UNSET))

        username = d.pop("username", UNSET)

        created_at = d.pop("created_at", UNSET)

        updated_at = d.pop("updated_at", UNSET)

        last_use_at = d.pop("last_use_at", UNSET)

        expires_at = d.pop("expires_at", UNSET)

        password = d.pop("password", UNSET)

        max_tokens = d.pop("max_tokens", UNSET)

        used_tokens = d.pop("used_tokens", UNSET)

        allow_from = cast(List[str], d.pop("allow_from", UNSET))

        share = cls(
            id=id,
            name=name,
            description=description,
            scope=scope,
            paths=paths,
            username=username,
            created_at=created_at,
            updated_at=updated_at,
            last_use_at=last_use_at,
            expires_at=expires_at,
            password=password,
            max_tokens=max_tokens,
            used_tokens=used_tokens,
            allow_from=allow_from,
        )

        share.additional_properties = d
        return share

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.azure_blob_fs_config_access_tier import AzureBlobFsConfigAccessTier
from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="AzureBlobFsConfig")


@attr.s(auto_attribs=True)
class AzureBlobFsConfig:
    """Azure Blob Storage configuration details

    Attributes:
        container (Union[Unset, str]):
        account_name (Union[Unset, str]): Storage Account Name, leave blank to use SAS URL
        account_key (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must
            provide a payload and set the status to "Plain". The encryption key and additional data will be generated
            automatically. If you set the status to "Redacted" the existig secret will be preserved
        sas_url (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must provide a
            payload and set the status to "Plain". The encryption key and additional data will be generated automatically.
            If you set the status to "Redacted" the existig secret will be preserved
        endpoint (Union[Unset, str]): optional endpoint. Default is "blob.core.windows.net". If you use the emulator the
            endpoint must include the protocol, for example "http://127.0.0.1:10000"
        upload_part_size (Union[Unset, int]): the buffer size (in MB) to use for multipart uploads. If this value is set
            to zero, the default value (5MB) will be used.
        upload_concurrency (Union[Unset, int]): the number of parts to upload in parallel. If this value is set to zero,
            the default value (5) will be used
        download_part_size (Union[Unset, int]): the buffer size (in MB) to use for multipart downloads. If this value is
            set to zero, the default value (5MB) will be used.
        download_concurrency (Union[Unset, int]): the number of parts to download in parallel. If this value is set to
            zero, the default value (5) will be used
        access_tier (Union[Unset, AzureBlobFsConfigAccessTier]):
        key_prefix (Union[Unset, str]): key_prefix is similar to a chroot directory for a local filesystem. If specified
            the user will only see contents that starts with this prefix and so you can restrict access to a specific
            virtual folder. The prefix, if not empty, must not start with "/" and must end with "/". If empty the whole
            container contents will be available Example: folder/subfolder/.
        use_emulator (Union[Unset, bool]):
    """

    container: Union[Unset, str] = UNSET
    account_name: Union[Unset, str] = UNSET
    account_key: Union[Unset, Secret] = UNSET
    sas_url: Union[Unset, Secret] = UNSET
    endpoint: Union[Unset, str] = UNSET
    upload_part_size: Union[Unset, int] = UNSET
    upload_concurrency: Union[Unset, int] = UNSET
    download_part_size: Union[Unset, int] = UNSET
    download_concurrency: Union[Unset, int] = UNSET
    access_tier: Union[Unset, AzureBlobFsConfigAccessTier] = UNSET
    key_prefix: Union[Unset, str] = UNSET
    use_emulator: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        container = self.container
        account_name = self.account_name
        account_key: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.account_key, Unset):
            account_key = self.account_key.to_dict()

        sas_url: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sas_url, Unset):
            sas_url = self.sas_url.to_dict()

        endpoint = self.endpoint
        upload_part_size = self.upload_part_size
        upload_concurrency = self.upload_concurrency
        download_part_size = self.download_part_size
        download_concurrency = self.download_concurrency
        access_tier: Union[Unset, str] = UNSET
        if not isinstance(self.access_tier, Unset):
            access_tier = self.access_tier.value

        key_prefix = self.key_prefix
        use_emulator = self.use_emulator

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if container is not UNSET:
            field_dict["container"] = container
        if account_name is not UNSET:
            field_dict["account_name"] = account_name
        if account_key is not UNSET:
            field_dict["account_key"] = account_key
        if sas_url is not UNSET:
            field_dict["sas_url"] = sas_url
        if endpoint is not UNSET:
            field_dict["endpoint"] = endpoint
        if upload_part_size is not UNSET:
            field_dict["upload_part_size"] = upload_part_size
        if upload_concurrency is not UNSET:
            field_dict["upload_concurrency"] = upload_concurrency
        if download_part_size is not UNSET:
            field_dict["download_part_size"] = download_part_size
        if download_concurrency is not UNSET:
            field_dict["download_concurrency"] = download_concurrency
        if access_tier is not UNSET:
            field_dict["access_tier"] = access_tier
        if key_prefix is not UNSET:
            field_dict["key_prefix"] = key_prefix
        if use_emulator is not UNSET:
            field_dict["use_emulator"] = use_emulator

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        container = d.pop("container", UNSET)

        account_name = d.pop("account_name", UNSET)

        _account_key = d.pop("account_key", UNSET)
        account_key: Union[Unset, Secret]
        if isinstance(_account_key, Unset):
            account_key = UNSET
        else:
            account_key = Secret.from_dict(_account_key)

        _sas_url = d.pop("sas_url", UNSET)
        sas_url: Union[Unset, Secret]
        if isinstance(_sas_url, Unset):
            sas_url = UNSET
        else:
            sas_url = Secret.from_dict(_sas_url)

        endpoint = d.pop("endpoint", UNSET)

        upload_part_size = d.pop("upload_part_size", UNSET)

        upload_concurrency = d.pop("upload_concurrency", UNSET)

        download_part_size = d.pop("download_part_size", UNSET)

        download_concurrency = d.pop("download_concurrency", UNSET)

        _access_tier = d.pop("access_tier", UNSET)
        access_tier: Union[Unset, AzureBlobFsConfigAccessTier]
        if isinstance(_access_tier, Unset):
            access_tier = UNSET
        else:
            access_tier = AzureBlobFsConfigAccessTier(_access_tier)

        key_prefix = d.pop("key_prefix", UNSET)

        use_emulator = d.pop("use_emulator", UNSET)

        azure_blob_fs_config = cls(
            container=container,
            account_name=account_name,
            account_key=account_key,
            sas_url=sas_url,
            endpoint=endpoint,
            upload_part_size=upload_part_size,
            upload_concurrency=upload_concurrency,
            download_part_size=download_part_size,
            download_concurrency=download_concurrency,
            access_tier=access_tier,
            key_prefix=key_prefix,
            use_emulator=use_emulator,
        )

        azure_blob_fs_config.additional_properties = d
        return azure_blob_fs_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.totp_config import TOTPConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="MFAStatus")


@attr.s(auto_attribs=True)
class MFAStatus:
    """
    Attributes:
        is_active (Union[Unset, bool]):
        totp_configs (Union[Unset, List[TOTPConfig]]):
    """

    is_active: Union[Unset, bool] = UNSET
    totp_configs: Union[Unset, List[TOTPConfig]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        is_active = self.is_active
        totp_configs: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.totp_configs, Unset):
            totp_configs = []
            for totp_configs_item_data in self.totp_configs:
                totp_configs_item = totp_configs_item_data.to_dict()

                totp_configs.append(totp_configs_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if is_active is not UNSET:
            field_dict["is_active"] = is_active
        if totp_configs is not UNSET:
            field_dict["totp_configs"] = totp_configs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        is_active = d.pop("is_active", UNSET)

        totp_configs = []
        _totp_configs = d.pop("totp_configs", UNSET)
        for totp_configs_item_data in _totp_configs or []:
            totp_configs_item = TOTPConfig.from_dict(totp_configs_item_data)

            totp_configs.append(totp_configs_item)

        mfa_status = cls(
            is_active=is_active,
            totp_configs=totp_configs,
        )

        mfa_status.additional_properties = d
        return mfa_status

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.tls_versions import TLSVersions
from ..types import UNSET, Unset

T = TypeVar("T", bound="WebDAVBinding")


@attr.s(auto_attribs=True)
class WebDAVBinding:
    """
    Attributes:
        address (Union[Unset, str]): TCP address the server listen on
        port (Union[Unset, int]): the port used for serving requests
        enable_https (Union[Unset, bool]):
        min_tls_version (Union[Unset, TLSVersions]): TLS version:
              * `12` - TLS 1.2
              * `13` - TLS 1.3
        client_auth_type (Union[Unset, int]): 1 means that client certificate authentication is required in addition to
            HTTP basic authentication
        tls_cipher_suites (Union[Unset, List[str]]): List of supported cipher suites for TLS version 1.2. If empty  a
            default list of secure cipher suites is used, with a preference order based on hardware performance
        prefix (Union[Unset, str]): Prefix for WebDAV resources, if empty WebDAV resources will be available at the `/`
            URI
        proxy_allowed (Union[Unset, List[str]]): List of IP addresses and IP ranges allowed to set proxy headers
    """

    address: Union[Unset, str] = UNSET
    port: Union[Unset, int] = UNSET
    enable_https: Union[Unset, bool] = UNSET
    min_tls_version: Union[Unset, TLSVersions] = UNSET
    client_auth_type: Union[Unset, int] = UNSET
    tls_cipher_suites: Union[Unset, List[str]] = UNSET
    prefix: Union[Unset, str] = UNSET
    proxy_allowed: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        address = self.address
        port = self.port
        enable_https = self.enable_https
        min_tls_version: Union[Unset, int] = UNSET
        if not isinstance(self.min_tls_version, Unset):
            min_tls_version = self.min_tls_version.value

        client_auth_type = self.client_auth_type
        tls_cipher_suites: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tls_cipher_suites, Unset):
            tls_cipher_suites = self.tls_cipher_suites

        prefix = self.prefix
        proxy_allowed: Union[Unset, List[str]] = UNSET
        if not isinstance(self.proxy_allowed, Unset):
            proxy_allowed = self.proxy_allowed

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address is not UNSET:
            field_dict["address"] = address
        if port is not UNSET:
            field_dict["port"] = port
        if enable_https is not UNSET:
            field_dict["enable_https"] = enable_https
        if min_tls_version is not UNSET:
            field_dict["min_tls_version"] = min_tls_version
        if client_auth_type is not UNSET:
            field_dict["client_auth_type"] = client_auth_type
        if tls_cipher_suites is not UNSET:
            field_dict["tls_cipher_suites"] = tls_cipher_suites
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if proxy_allowed is not UNSET:
            field_dict["proxy_allowed"] = proxy_allowed

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        address = d.pop("address", UNSET)

        port = d.pop("port", UNSET)

        enable_https = d.pop("enable_https", UNSET)

        _min_tls_version = d.pop("min_tls_version", UNSET)
        min_tls_version: Union[Unset, TLSVersions]
        if isinstance(_min_tls_version, Unset):
            min_tls_version = UNSET
        else:
            min_tls_version = TLSVersions(_min_tls_version)

        client_auth_type = d.pop("client_auth_type", UNSET)

        tls_cipher_suites = cast(List[str], d.pop("tls_cipher_suites", UNSET))

        prefix = d.pop("prefix", UNSET)

        proxy_allowed = cast(List[str], d.pop("proxy_allowed", UNSET))

        web_dav_binding = cls(
            address=address,
            port=port,
            enable_https=enable_https,
            min_tls_version=min_tls_version,
            client_auth_type=client_auth_type,
            tls_cipher_suites=tls_cipher_suites,
            prefix=prefix,
            proxy_allowed=proxy_allowed,
        )

        web_dav_binding.additional_properties = d
        return web_dav_binding

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.ftpd_binding_active_connections_security import (
    FTPDBindingActiveConnectionsSecurity,
)
from ..models.ftpd_binding_passive_connections_security import (
    FTPDBindingPassiveConnectionsSecurity,
)
from ..models.ftpd_binding_tls_mode import FTPDBindingTlsMode
from ..models.passive_ip_override import PassiveIPOverride
from ..models.tls_versions import TLSVersions
from ..types import UNSET, Unset

T = TypeVar("T", bound="FTPDBinding")


@attr.s(auto_attribs=True)
class FTPDBinding:
    """
    Attributes:
        address (Union[Unset, str]): TCP address the server listen on
        port (Union[Unset, int]): the port used for serving requests
        apply_proxy_config (Union[Unset, bool]): apply the proxy configuration, if any
        tls_mode (Union[Unset, FTPDBindingTlsMode]): TLS mode:
              * `0` - clear or explicit TLS
              * `1` - explicit TLS required
              * `2` - implicit TLS
        min_tls_version (Union[Unset, TLSVersions]): TLS version:
              * `12` - TLS 1.2
              * `13` - TLS 1.3
        force_passive_ip (Union[Unset, str]): External IP address to expose for passive connections
        passive_ip_overrides (Union[Unset, List[PassiveIPOverride]]):
        client_auth_type (Union[Unset, int]): 1 means that client certificate authentication is required in addition to
            FTP authentication
        tls_cipher_suites (Union[Unset, List[str]]): List of supported cipher suites for TLS version 1.2. If empty  a
            default list of secure cipher suites is used, with a preference order based on hardware performance
        passive_connections_security (Union[Unset, FTPDBindingPassiveConnectionsSecurity]): Active connections security:
              * `0` - require matching peer IP addresses of control and data connection
              * `1` - disable any checks
        active_connections_security (Union[Unset, FTPDBindingActiveConnectionsSecurity]): Active connections security:
              * `0` - require matching peer IP addresses of control and data connection
              * `1` - disable any checks
        debug (Union[Unset, bool]): If enabled any FTP command will be logged
    """

    address: Union[Unset, str] = UNSET
    port: Union[Unset, int] = UNSET
    apply_proxy_config: Union[Unset, bool] = UNSET
    tls_mode: Union[Unset, FTPDBindingTlsMode] = UNSET
    min_tls_version: Union[Unset, TLSVersions] = UNSET
    force_passive_ip: Union[Unset, str] = UNSET
    passive_ip_overrides: Union[Unset, List[PassiveIPOverride]] = UNSET
    client_auth_type: Union[Unset, int] = UNSET
    tls_cipher_suites: Union[Unset, List[str]] = UNSET
    passive_connections_security: Union[
        Unset, FTPDBindingPassiveConnectionsSecurity
    ] = UNSET
    active_connections_security: Union[
        Unset, FTPDBindingActiveConnectionsSecurity
    ] = UNSET
    debug: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        address = self.address
        port = self.port
        apply_proxy_config = self.apply_proxy_config
        tls_mode: Union[Unset, int] = UNSET
        if not isinstance(self.tls_mode, Unset):
            tls_mode = self.tls_mode.value

        min_tls_version: Union[Unset, int] = UNSET
        if not isinstance(self.min_tls_version, Unset):
            min_tls_version = self.min_tls_version.value

        force_passive_ip = self.force_passive_ip
        passive_ip_overrides: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.passive_ip_overrides, Unset):
            passive_ip_overrides = []
            for passive_ip_overrides_item_data in self.passive_ip_overrides:
                passive_ip_overrides_item = passive_ip_overrides_item_data.to_dict()

                passive_ip_overrides.append(passive_ip_overrides_item)

        client_auth_type = self.client_auth_type
        tls_cipher_suites: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tls_cipher_suites, Unset):
            tls_cipher_suites = self.tls_cipher_suites

        passive_connections_security: Union[Unset, int] = UNSET
        if not isinstance(self.passive_connections_security, Unset):
            passive_connections_security = self.passive_connections_security.value

        active_connections_security: Union[Unset, int] = UNSET
        if not isinstance(self.active_connections_security, Unset):
            active_connections_security = self.active_connections_security.value

        debug = self.debug

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if address is not UNSET:
            field_dict["address"] = address
        if port is not UNSET:
            field_dict["port"] = port
        if apply_proxy_config is not UNSET:
            field_dict["apply_proxy_config"] = apply_proxy_config
        if tls_mode is not UNSET:
            field_dict["tls_mode"] = tls_mode
        if min_tls_version is not UNSET:
            field_dict["min_tls_version"] = min_tls_version
        if force_passive_ip is not UNSET:
            field_dict["force_passive_ip"] = force_passive_ip
        if passive_ip_overrides is not UNSET:
            field_dict["passive_ip_overrides"] = passive_ip_overrides
        if client_auth_type is not UNSET:
            field_dict["client_auth_type"] = client_auth_type
        if tls_cipher_suites is not UNSET:
            field_dict["tls_cipher_suites"] = tls_cipher_suites
        if passive_connections_security is not UNSET:
            field_dict["passive_connections_security"] = passive_connections_security
        if active_connections_security is not UNSET:
            field_dict["active_connections_security"] = active_connections_security
        if debug is not UNSET:
            field_dict["debug"] = debug

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        address = d.pop("address", UNSET)

        port = d.pop("port", UNSET)

        apply_proxy_config = d.pop("apply_proxy_config", UNSET)

        _tls_mode = d.pop("tls_mode", UNSET)
        tls_mode: Union[Unset, FTPDBindingTlsMode]
        if isinstance(_tls_mode, Unset):
            tls_mode = UNSET
        else:
            tls_mode = FTPDBindingTlsMode(_tls_mode)

        _min_tls_version = d.pop("min_tls_version", UNSET)
        min_tls_version: Union[Unset, TLSVersions]
        if isinstance(_min_tls_version, Unset):
            min_tls_version = UNSET
        else:
            min_tls_version = TLSVersions(_min_tls_version)

        force_passive_ip = d.pop("force_passive_ip", UNSET)

        passive_ip_overrides = []
        _passive_ip_overrides = d.pop("passive_ip_overrides", UNSET)
        for passive_ip_overrides_item_data in _passive_ip_overrides or []:
            passive_ip_overrides_item = PassiveIPOverride.from_dict(
                passive_ip_overrides_item_data
            )

            passive_ip_overrides.append(passive_ip_overrides_item)

        client_auth_type = d.pop("client_auth_type", UNSET)

        tls_cipher_suites = cast(List[str], d.pop("tls_cipher_suites", UNSET))

        _passive_connections_security = d.pop("passive_connections_security", UNSET)
        passive_connections_security: Union[
            Unset, FTPDBindingPassiveConnectionsSecurity
        ]
        if isinstance(_passive_connections_security, Unset):
            passive_connections_security = UNSET
        else:
            passive_connections_security = FTPDBindingPassiveConnectionsSecurity(
                _passive_connections_security
            )

        _active_connections_security = d.pop("active_connections_security", UNSET)
        active_connections_security: Union[Unset, FTPDBindingActiveConnectionsSecurity]
        if isinstance(_active_connections_security, Unset):
            active_connections_security = UNSET
        else:
            active_connections_security = FTPDBindingActiveConnectionsSecurity(
                _active_connections_security
            )

        debug = d.pop("debug", UNSET)

        ftpd_binding = cls(
            address=address,
            port=port,
            apply_proxy_config=apply_proxy_config,
            tls_mode=tls_mode,
            min_tls_version=min_tls_version,
            force_passive_ip=force_passive_ip,
            passive_ip_overrides=passive_ip_overrides,
            client_auth_type=client_auth_type,
            tls_cipher_suites=tls_cipher_suites,
            passive_connections_security=passive_connections_security,
            active_connections_security=active_connections_security,
            debug=debug,
        )

        ftpd_binding.additional_properties = d
        return ftpd_binding

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.filesystem_config import FilesystemConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="VirtualFolder")


@attr.s(auto_attribs=True)
class VirtualFolder:
    """A virtual folder is a mapping between a SFTPGo virtual path and a filesystem path outside the user home directory.
    The specified paths must be absolute and the virtual path cannot be "/", it must be a sub directory. The parent
    directory for the specified virtual path must exist. SFTPGo will try to automatically create any missing parent
    directory for the configured virtual folders at user login.

        Attributes:
            virtual_path (str):
            id (Union[Unset, int]):
            name (Union[Unset, str]): unique name for this virtual folder
            mapped_path (Union[Unset, str]): absolute filesystem path to use as virtual folder
            description (Union[Unset, str]): optional description
            used_quota_size (Union[Unset, int]):
            used_quota_files (Union[Unset, int]):
            last_quota_update (Union[Unset, int]): Last quota update as unix timestamp in milliseconds
            users (Union[Unset, List[str]]): list of usernames associated with this virtual folder
            filesystem (Union[Unset, FilesystemConfig]): Storage filesystem details
            quota_size (Union[Unset, int]): Quota as size in bytes. 0 menas unlimited, -1 means included in user quota.
                Please note that quota is updated if files are added/removed via SFTPGo otherwise a quota scan or a manual quota
                update is needed
            quota_files (Union[Unset, int]): Quota as number of files. 0 menas unlimited, , -1 means included in user quota.
                Please note that quota is updated if files are added/removed via SFTPGo otherwise a quota scan or a manual quota
                update is needed
    """

    virtual_path: str
    id: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    mapped_path: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    used_quota_size: Union[Unset, int] = UNSET
    used_quota_files: Union[Unset, int] = UNSET
    last_quota_update: Union[Unset, int] = UNSET
    users: Union[Unset, List[str]] = UNSET
    filesystem: Union[Unset, FilesystemConfig] = UNSET
    quota_size: Union[Unset, int] = UNSET
    quota_files: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        virtual_path = self.virtual_path
        id = self.id
        name = self.name
        mapped_path = self.mapped_path
        description = self.description
        used_quota_size = self.used_quota_size
        used_quota_files = self.used_quota_files
        last_quota_update = self.last_quota_update
        users: Union[Unset, List[str]] = UNSET
        if not isinstance(self.users, Unset):
            users = self.users

        filesystem: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.filesystem, Unset):
            filesystem = self.filesystem.to_dict()

        quota_size = self.quota_size
        quota_files = self.quota_files

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "virtual_path": virtual_path,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if mapped_path is not UNSET:
            field_dict["mapped_path"] = mapped_path
        if description is not UNSET:
            field_dict["description"] = description
        if used_quota_size is not UNSET:
            field_dict["used_quota_size"] = used_quota_size
        if used_quota_files is not UNSET:
            field_dict["used_quota_files"] = used_quota_files
        if last_quota_update is not UNSET:
            field_dict["last_quota_update"] = last_quota_update
        if users is not UNSET:
            field_dict["users"] = users
        if filesystem is not UNSET:
            field_dict["filesystem"] = filesystem
        if quota_size is not UNSET:
            field_dict["quota_size"] = quota_size
        if quota_files is not UNSET:
            field_dict["quota_files"] = quota_files

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        virtual_path = d.pop("virtual_path")

        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        mapped_path = d.pop("mapped_path", UNSET)

        description = d.pop("description", UNSET)

        used_quota_size = d.pop("used_quota_size", UNSET)

        used_quota_files = d.pop("used_quota_files", UNSET)

        last_quota_update = d.pop("last_quota_update", UNSET)

        users = cast(List[str], d.pop("users", UNSET))

        _filesystem = d.pop("filesystem", UNSET)
        filesystem: Union[Unset, FilesystemConfig]
        if isinstance(_filesystem, Unset):
            filesystem = UNSET
        else:
            filesystem = FilesystemConfig.from_dict(_filesystem)

        quota_size = d.pop("quota_size", UNSET)

        quota_files = d.pop("quota_files", UNSET)

        virtual_folder = cls(
            virtual_path=virtual_path,
            id=id,
            name=name,
            mapped_path=mapped_path,
            description=description,
            used_quota_size=used_quota_size,
            used_quota_files=used_quota_files,
            last_quota_update=last_quota_update,
            users=users,
            filesystem=filesystem,
            quota_size=quota_size,
            quota_files=quota_files,
        )

        virtual_folder.additional_properties = d
        return virtual_folder

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.gcs_config_automatic_credentials import GCSConfigAutomaticCredentials
from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="GCSConfig")


@attr.s(auto_attribs=True)
class GCSConfig:
    """Google Cloud Storage configuration details. The "credentials" field must be populated only when adding/updating a
    user. It will be always omitted, since there are sensitive data, when you search/get users

        Attributes:
            bucket (Union[Unset, str]):
            credentials (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must
                provide a payload and set the status to "Plain". The encryption key and additional data will be generated
                automatically. If you set the status to "Redacted" the existig secret will be preserved
            automatic_credentials (Union[Unset, GCSConfigAutomaticCredentials]): Automatic credentials:
                  * `0` - disabled, explicit credentials, using a JSON credentials file, must be provided. This is the default
                value if the field is null
                  * `1` - enabled, we try to use the Application Default Credentials (ADC) strategy to find your application's
                credentials
            storage_class (Union[Unset, str]):
            acl (Union[Unset, str]): The ACL to apply to uploaded objects. Leave empty to use the default ACL. For more
                information and available ACLs, refer to the JSON API here: https://cloud.google.com/storage/docs/access-
                control/lists#predefined-acl
            key_prefix (Union[Unset, str]): key_prefix is similar to a chroot directory for a local filesystem. If specified
                the user will only see contents that starts with this prefix and so you can restrict access to a specific
                virtual folder. The prefix, if not empty, must not start with "/" and must end with "/". If empty the whole
                bucket contents will be available Example: folder/subfolder/.
    """

    bucket: Union[Unset, str] = UNSET
    credentials: Union[Unset, Secret] = UNSET
    automatic_credentials: Union[Unset, GCSConfigAutomaticCredentials] = UNSET
    storage_class: Union[Unset, str] = UNSET
    acl: Union[Unset, str] = UNSET
    key_prefix: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        bucket = self.bucket
        credentials: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.credentials, Unset):
            credentials = self.credentials.to_dict()

        automatic_credentials: Union[Unset, int] = UNSET
        if not isinstance(self.automatic_credentials, Unset):
            automatic_credentials = self.automatic_credentials.value

        storage_class = self.storage_class
        acl = self.acl
        key_prefix = self.key_prefix

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if bucket is not UNSET:
            field_dict["bucket"] = bucket
        if credentials is not UNSET:
            field_dict["credentials"] = credentials
        if automatic_credentials is not UNSET:
            field_dict["automatic_credentials"] = automatic_credentials
        if storage_class is not UNSET:
            field_dict["storage_class"] = storage_class
        if acl is not UNSET:
            field_dict["acl"] = acl
        if key_prefix is not UNSET:
            field_dict["key_prefix"] = key_prefix

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bucket = d.pop("bucket", UNSET)

        _credentials = d.pop("credentials", UNSET)
        credentials: Union[Unset, Secret]
        if isinstance(_credentials, Unset):
            credentials = UNSET
        else:
            credentials = Secret.from_dict(_credentials)

        _automatic_credentials = d.pop("automatic_credentials", UNSET)
        automatic_credentials: Union[Unset, GCSConfigAutomaticCredentials]
        if isinstance(_automatic_credentials, Unset):
            automatic_credentials = UNSET
        else:
            automatic_credentials = GCSConfigAutomaticCredentials(
                _automatic_credentials
            )

        storage_class = d.pop("storage_class", UNSET)

        acl = d.pop("acl", UNSET)

        key_prefix = d.pop("key_prefix", UNSET)

        gcs_config = cls(
            bucket=bucket,
            credentials=credentials,
            automatic_credentials=automatic_credentials,
            storage_class=storage_class,
            acl=acl,
            key_prefix=key_prefix,
        )

        gcs_config.additional_properties = d
        return gcs_config

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.api_response import ApiResponse
from ...models.backup_data import BackupData
from ...models.dumpdata_indent import DumpdataIndent
from ...models.dumpdata_output_data import DumpdataOutputData
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    output_file: Union[Unset, None, str] = UNSET,
    output_data: Union[Unset, None, DumpdataOutputData] = UNSET,
    indent: Union[Unset, None, DumpdataIndent] = UNSET,
) -> Dict[str, Any]:
    url = "{}/dumpdata".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["output-file"] = output_file

    json_output_data: Union[Unset, None, int] = UNSET
    if not isinstance(output_data, Unset):
        json_output_data = output_data.value if output_data else None

    params["output-data"] = json_output_data

    json_indent: Union[Unset, None, int] = UNSET
    if not isinstance(indent, Unset):
        json_indent = indent.value if indent else None

    params["indent"] = json_indent

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(
    *, response: httpx.Response
) -> Optional[Union[Any, Union[ApiResponse, BackupData]]]:
    if response.status_code == 200:

        def _parse_response_200(data: object) -> Union[ApiResponse, BackupData]:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                response_200_type_0 = ApiResponse.from_dict(data)

                return response_200_type_0
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            response_200_type_1 = BackupData.from_dict(data)

            return response_200_type_1

        response_200 = _parse_response_200(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(
    *, response: httpx.Response
) -> Response[Union[Any, Union[ApiResponse, BackupData]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    output_file: Union[Unset, None, str] = UNSET,
    output_data: Union[Unset, None, DumpdataOutputData] = UNSET,
    indent: Union[Unset, None, DumpdataIndent] = UNSET,
) -> Response[Union[Any, Union[ApiResponse, BackupData]]]:
    """Dump data

     Backups data as data provider independent JSON. The backup can be saved in a local file on the
    server, to avoid exposing sensitive data over the network, or returned as response body. The output
    of dumpdata can be used as input for loaddata

    Args:
        output_file (Union[Unset, None, str]):
        output_data (Union[Unset, None, DumpdataOutputData]):
        indent (Union[Unset, None, DumpdataIndent]):

    Returns:
        Response[Union[Any, Union[ApiResponse, BackupData]]]
    """

    kwargs = _get_kwargs(
        client=client,
        output_file=output_file,
        output_data=output_data,
        indent=indent,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    output_file: Union[Unset, None, str] = UNSET,
    output_data: Union[Unset, None, DumpdataOutputData] = UNSET,
    indent: Union[Unset, None, DumpdataIndent] = UNSET,
) -> Optional[Union[Any, Union[ApiResponse, BackupData]]]:
    """Dump data

     Backups data as data provider independent JSON. The backup can be saved in a local file on the
    server, to avoid exposing sensitive data over the network, or returned as response body. The output
    of dumpdata can be used as input for loaddata

    Args:
        output_file (Union[Unset, None, str]):
        output_data (Union[Unset, None, DumpdataOutputData]):
        indent (Union[Unset, None, DumpdataIndent]):

    Returns:
        Response[Union[Any, Union[ApiResponse, BackupData]]]
    """

    return sync_detailed(
        client=client,
        output_file=output_file,
        output_data=output_data,
        indent=indent,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    output_file: Union[Unset, None, str] = UNSET,
    output_data: Union[Unset, None, DumpdataOutputData] = UNSET,
    indent: Union[Unset, None, DumpdataIndent] = UNSET,
) -> Response[Union[Any, Union[ApiResponse, BackupData]]]:
    """Dump data

     Backups data as data provider independent JSON. The backup can be saved in a local file on the
    server, to avoid exposing sensitive data over the network, or returned as response body. The output
    of dumpdata can be used as input for loaddata

    Args:
        output_file (Union[Unset, None, str]):
        output_data (Union[Unset, None, DumpdataOutputData]):
        indent (Union[Unset, None, DumpdataIndent]):

    Returns:
        Response[Union[Any, Union[ApiResponse, BackupData]]]
    """

    kwargs = _get_kwargs(
        client=client,
        output_file=output_file,
        output_data=output_data,
        indent=indent,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    output_file: Union[Unset, None, str] = UNSET,
    output_data: Union[Unset, None, DumpdataOutputData] = UNSET,
    indent: Union[Unset, None, DumpdataIndent] = UNSET,
) -> Optional[Union[Any, Union[ApiResponse, BackupData]]]:
    """Dump data

     Backups data as data provider independent JSON. The backup can be saved in a local file on the
    server, to avoid exposing sensitive data over the network, or returned as response body. The output
    of dumpdata can be used as input for loaddata

    Args:
        output_file (Union[Unset, None, str]):
        output_data (Union[Unset, None, DumpdataOutputData]):
        indent (Union[Unset, None, DumpdataIndent]):

    Returns:
        Response[Union[Any, Union[ApiResponse, BackupData]]]
    """

    return (
        await asyncio_detailed(
            client=client,
            output_file=output_file,
            output_data=output_data,
            indent=indent,
        )
    ).parsed



from typing import Any, Dict, Optional, Union, cast

import httpx

from ...client import Client
from ...models.api_response import ApiResponse
from ...models.folder_quota_update_usage_mode import FolderQuotaUpdateUsageMode
from ...models.quota_usage import QuotaUsage
from ...types import UNSET, Response, Unset


def _get_kwargs(
    name: str,
    *,
    client: Client,
    json_body: QuotaUsage,
    mode: Union[Unset, None, FolderQuotaUpdateUsageMode] = UNSET,
) -> Dict[str, Any]:
    url = "{}/quotas/folders/{name}/usage".format(client.base_url, name=name)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    json_mode: Union[Unset, None, str] = UNSET
    if not isinstance(mode, Unset):
        json_mode = mode.value if mode else None

    params["mode"] = json_mode

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    json_json_body = json_body.to_dict()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, ApiResponse]]:
    if response.status_code == 200:
        response_200 = ApiResponse.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 404:
        response_404 = cast(Any, None)
        return response_404
    if response.status_code == 409:
        response_409 = cast(Any, None)
        return response_409
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, ApiResponse]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    name: str,
    *,
    client: Client,
    json_body: QuotaUsage,
    mode: Union[Unset, None, FolderQuotaUpdateUsageMode] = UNSET,
) -> Response[Union[Any, ApiResponse]]:
    """Update folder quota usage limits

     Sets the current used quota limits for the given folder

    Args:
        name (str):
        mode (Union[Unset, None, FolderQuotaUpdateUsageMode]): Update type:
              * `add` - add the specified quota limits to the current used ones
              * `reset` - reset the values to the specified ones. This is the default
             Example: reset.
        json_body (QuotaUsage):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    kwargs = _get_kwargs(
        name=name,
        client=client,
        json_body=json_body,
        mode=mode,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    name: str,
    *,
    client: Client,
    json_body: QuotaUsage,
    mode: Union[Unset, None, FolderQuotaUpdateUsageMode] = UNSET,
) -> Optional[Union[Any, ApiResponse]]:
    """Update folder quota usage limits

     Sets the current used quota limits for the given folder

    Args:
        name (str):
        mode (Union[Unset, None, FolderQuotaUpdateUsageMode]): Update type:
              * `add` - add the specified quota limits to the current used ones
              * `reset` - reset the values to the specified ones. This is the default
             Example: reset.
        json_body (QuotaUsage):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    return sync_detailed(
        name=name,
        client=client,
        json_body=json_body,
        mode=mode,
    ).parsed


async def asyncio_detailed(
    name: str,
    *,
    client: Client,
    json_body: QuotaUsage,
    mode: Union[Unset, None, FolderQuotaUpdateUsageMode] = UNSET,
) -> Response[Union[Any, ApiResponse]]:
    """Update folder quota usage limits

     Sets the current used quota limits for the given folder

    Args:
        name (str):
        mode (Union[Unset, None, FolderQuotaUpdateUsageMode]): Update type:
              * `add` - add the specified quota limits to the current used ones
              * `reset` - reset the values to the specified ones. This is the default
             Example: reset.
        json_body (QuotaUsage):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    kwargs = _get_kwargs(
        name=name,
        client=client,
        json_body=json_body,
        mode=mode,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    name: str,
    *,
    client: Client,
    json_body: QuotaUsage,
    mode: Union[Unset, None, FolderQuotaUpdateUsageMode] = UNSET,
) -> Optional[Union[Any, ApiResponse]]:
    """Update folder quota usage limits

     Sets the current used quota limits for the given folder

    Args:
        name (str):
        mode (Union[Unset, None, FolderQuotaUpdateUsageMode]): Update type:
              * `add` - add the specified quota limits to the current used ones
              * `reset` - reset the values to the specified ones. This is the default
             Example: reset.
        json_body (QuotaUsage):

    Returns:
        Response[Union[Any, ApiResponse]]
    """

    return (
        await asyncio_detailed(
            name=name,
            client=client,
            json_body=json_body,
            mode=mode,
        )
    ).parsed



from typing import Any, Dict, Union

import httpx

from ...client import AuthenticatedClient
from ...types import UNSET, Response, Unset


def _get_kwargs(
    id: str,
    *,
    client: AuthenticatedClient,
    compress: Union[Unset, None, bool] = True,
) -> Dict[str, Any]:
    url = "{}/shares/{id}".format(client.base_url, id=id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["compress"] = compress

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _build_response(*, response: httpx.Response) -> Response[Any]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=None,
    )


def sync_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    compress: Union[Unset, None, bool] = True,
) -> Response[Any]:
    """Download shared files and folders as a single zip file

     A zip file, containing the shared files and folders, will be generated on the fly and returned as
    response body. Only folders and regular files will be included in the zip. The share must be defined
    with the read scope and the associated user must have list and download permissions

    Args:
        id (str):
        compress (Union[Unset, None, bool]):  Default: True.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        compress=compress,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


async def asyncio_detailed(
    id: str,
    *,
    client: AuthenticatedClient,
    compress: Union[Unset, None, bool] = True,
) -> Response[Any]:
    """Download shared files and folders as a single zip file

     A zip file, containing the shared files and folders, will be generated on the fly and returned as
    response body. Only folders and regular files will be included in the zip. The share must be defined
    with the read scope and the associated user must have list and download permissions

    Args:
        id (str):
        compress (Union[Unset, None, bool]):  Default: True.

    Returns:
        Response[Any]
    """

    kwargs = _get_kwargs(
        id=id,
        client=client,
        compress=compress,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="HooksFilter")


@attr.s(auto_attribs=True)
class HooksFilter:
    """User specific hook overrides

    Attributes:
        external_auth_disabled (Union[Unset, bool]): If true, the external auth hook, if defined, will not be executed
        pre_login_disabled (Union[Unset, bool]): If true, the pre-login hook, if defined, will not be executed
        check_password_disabled (Union[Unset, bool]): If true, the check password hook, if defined, will not be executed
    """

    external_auth_disabled: Union[Unset, bool] = UNSET
    pre_login_disabled: Union[Unset, bool] = UNSET
    check_password_disabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        external_auth_disabled = self.external_auth_disabled
        pre_login_disabled = self.pre_login_disabled
        check_password_disabled = self.check_password_disabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if external_auth_disabled is not UNSET:
            field_dict["external_auth_disabled"] = external_auth_disabled
        if pre_login_disabled is not UNSET:
            field_dict["pre_login_disabled"] = pre_login_disabled
        if check_password_disabled is not UNSET:
            field_dict["check_password_disabled"] = check_password_disabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        external_auth_disabled = d.pop("external_auth_disabled", UNSET)

        pre_login_disabled = d.pop("pre_login_disabled", UNSET)

        check_password_disabled = d.pop("check_password_disabled", UNSET)

        hooks_filter = cls(
            external_auth_disabled=external_auth_disabled,
            pre_login_disabled=pre_login_disabled,
            check_password_disabled=check_password_disabled,
        )

        hooks_filter.additional_properties = d
        return hooks_filter

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union, cast

import httpx

from ...client import Client
from ...models.event_protocols import EventProtocols
from ...models.fs_event import FsEvent
from ...models.fs_event_action import FsEventAction
from ...models.fs_event_status import FsEventStatus
from ...models.fs_providers import FsProviders
from ...models.get_fs_events_order import GetFsEventsOrder
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    start_timestamp: Union[Unset, None, int] = 0,
    end_timestamp: Union[Unset, None, int] = 0,
    actions: Union[Unset, None, List[FsEventAction]] = UNSET,
    username: Union[Unset, None, str] = UNSET,
    ip: Union[Unset, None, str] = UNSET,
    ssh_cmd: Union[Unset, None, str] = UNSET,
    fs_provider: Union[Unset, None, FsProviders] = UNSET,
    bucket: Union[Unset, None, str] = UNSET,
    endpoint: Union[Unset, None, str] = UNSET,
    protocols: Union[Unset, None, List[EventProtocols]] = UNSET,
    statuses: Union[Unset, None, List[FsEventStatus]] = UNSET,
    instance_ids: Union[Unset, None, List[str]] = UNSET,
    exclude_ids: Union[Unset, None, List[str]] = UNSET,
    limit: Union[Unset, None, int] = 100,
    order: Union[Unset, None, GetFsEventsOrder] = UNSET,
) -> Dict[str, Any]:
    url = "{}/events/fs".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    params: Dict[str, Any] = {}
    params["start_timestamp"] = start_timestamp

    params["end_timestamp"] = end_timestamp

    json_actions: Union[Unset, None, List[str]] = UNSET
    if not isinstance(actions, Unset):
        if actions is None:
            json_actions = None
        else:
            json_actions = []
            for actions_item_data in actions:
                actions_item = actions_item_data.value

                json_actions.append(actions_item)

    params["actions"] = json_actions

    params["username"] = username

    params["ip"] = ip

    params["ssh_cmd"] = ssh_cmd

    json_fs_provider: Union[Unset, None, int] = UNSET
    if not isinstance(fs_provider, Unset):
        json_fs_provider = fs_provider.value if fs_provider else None

    params["fs_provider"] = json_fs_provider

    params["bucket"] = bucket

    params["endpoint"] = endpoint

    json_protocols: Union[Unset, None, List[str]] = UNSET
    if not isinstance(protocols, Unset):
        if protocols is None:
            json_protocols = None
        else:
            json_protocols = []
            for protocols_item_data in protocols:
                protocols_item = protocols_item_data.value

                json_protocols.append(protocols_item)

    params["protocols"] = json_protocols

    json_statuses: Union[Unset, None, List[int]] = UNSET
    if not isinstance(statuses, Unset):
        if statuses is None:
            json_statuses = None
        else:
            json_statuses = []
            for statuses_item_data in statuses:
                statuses_item = statuses_item_data.value

                json_statuses.append(statuses_item)

    params["statuses"] = json_statuses

    json_instance_ids: Union[Unset, None, List[str]] = UNSET
    if not isinstance(instance_ids, Unset):
        if instance_ids is None:
            json_instance_ids = None
        else:
            json_instance_ids = instance_ids

    params["instance_ids"] = json_instance_ids

    json_exclude_ids: Union[Unset, None, List[str]] = UNSET
    if not isinstance(exclude_ids, Unset):
        if exclude_ids is None:
            json_exclude_ids = None
        else:
            json_exclude_ids = exclude_ids

    params["exclude_ids"] = json_exclude_ids

    params["limit"] = limit

    json_order: Union[Unset, None, str] = UNSET
    if not isinstance(order, Unset):
        json_order = order.value if order else None

    params["order"] = json_order

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    return {
        "method": "get",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[Any, List[FsEvent]]]:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = FsEvent.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200
    if response.status_code == 400:
        response_400 = cast(Any, None)
        return response_400
    if response.status_code == 401:
        response_401 = cast(Any, None)
        return response_401
    if response.status_code == 403:
        response_403 = cast(Any, None)
        return response_403
    if response.status_code == 500:
        response_500 = cast(Any, None)
        return response_500
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[Any, List[FsEvent]]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    start_timestamp: Union[Unset, None, int] = 0,
    end_timestamp: Union[Unset, None, int] = 0,
    actions: Union[Unset, None, List[FsEventAction]] = UNSET,
    username: Union[Unset, None, str] = UNSET,
    ip: Union[Unset, None, str] = UNSET,
    ssh_cmd: Union[Unset, None, str] = UNSET,
    fs_provider: Union[Unset, None, FsProviders] = UNSET,
    bucket: Union[Unset, None, str] = UNSET,
    endpoint: Union[Unset, None, str] = UNSET,
    protocols: Union[Unset, None, List[EventProtocols]] = UNSET,
    statuses: Union[Unset, None, List[FsEventStatus]] = UNSET,
    instance_ids: Union[Unset, None, List[str]] = UNSET,
    exclude_ids: Union[Unset, None, List[str]] = UNSET,
    limit: Union[Unset, None, int] = 100,
    order: Union[Unset, None, GetFsEventsOrder] = UNSET,
) -> Response[Union[Any, List[FsEvent]]]:
    """Get filesystem events

     Returns an array with one or more filesystem events applying the specified filters. This API is only
    available if you configure an \"eventsearcher\" plugin

    Args:
        start_timestamp (Union[Unset, None, int]):
        end_timestamp (Union[Unset, None, int]):
        actions (Union[Unset, None, List[FsEventAction]]):
        username (Union[Unset, None, str]):
        ip (Union[Unset, None, str]):
        ssh_cmd (Union[Unset, None, str]):
        fs_provider (Union[Unset, None, FsProviders]): Filesystem providers:
              * `0` - Local filesystem
              * `1` - S3 Compatible Object Storage
              * `2` - Google Cloud Storage
              * `3` - Azure Blob Storage
              * `4` - Local filesystem encrypted
              * `5` - SFTP
        bucket (Union[Unset, None, str]):
        endpoint (Union[Unset, None, str]):
        protocols (Union[Unset, None, List[EventProtocols]]):
        statuses (Union[Unset, None, List[FsEventStatus]]):
        instance_ids (Union[Unset, None, List[str]]):
        exclude_ids (Union[Unset, None, List[str]]):
        limit (Union[Unset, None, int]):  Default: 100.
        order (Union[Unset, None, GetFsEventsOrder]):  Example: DESC.

    Returns:
        Response[Union[Any, List[FsEvent]]]
    """

    kwargs = _get_kwargs(
        client=client,
        start_timestamp=start_timestamp,
        end_timestamp=end_timestamp,
        actions=actions,
        username=username,
        ip=ip,
        ssh_cmd=ssh_cmd,
        fs_provider=fs_provider,
        bucket=bucket,
        endpoint=endpoint,
        protocols=protocols,
        statuses=statuses,
        instance_ids=instance_ids,
        exclude_ids=exclude_ids,
        limit=limit,
        order=order,
    )

    response = httpx.request(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    start_timestamp: Union[Unset, None, int] = 0,
    end_timestamp: Union[Unset, None, int] = 0,
    actions: Union[Unset, None, List[FsEventAction]] = UNSET,
    username: Union[Unset, None, str] = UNSET,
    ip: Union[Unset, None, str] = UNSET,
    ssh_cmd: Union[Unset, None, str] = UNSET,
    fs_provider: Union[Unset, None, FsProviders] = UNSET,
    bucket: Union[Unset, None, str] = UNSET,
    endpoint: Union[Unset, None, str] = UNSET,
    protocols: Union[Unset, None, List[EventProtocols]] = UNSET,
    statuses: Union[Unset, None, List[FsEventStatus]] = UNSET,
    instance_ids: Union[Unset, None, List[str]] = UNSET,
    exclude_ids: Union[Unset, None, List[str]] = UNSET,
    limit: Union[Unset, None, int] = 100,
    order: Union[Unset, None, GetFsEventsOrder] = UNSET,
) -> Optional[Union[Any, List[FsEvent]]]:
    """Get filesystem events

     Returns an array with one or more filesystem events applying the specified filters. This API is only
    available if you configure an \"eventsearcher\" plugin

    Args:
        start_timestamp (Union[Unset, None, int]):
        end_timestamp (Union[Unset, None, int]):
        actions (Union[Unset, None, List[FsEventAction]]):
        username (Union[Unset, None, str]):
        ip (Union[Unset, None, str]):
        ssh_cmd (Union[Unset, None, str]):
        fs_provider (Union[Unset, None, FsProviders]): Filesystem providers:
              * `0` - Local filesystem
              * `1` - S3 Compatible Object Storage
              * `2` - Google Cloud Storage
              * `3` - Azure Blob Storage
              * `4` - Local filesystem encrypted
              * `5` - SFTP
        bucket (Union[Unset, None, str]):
        endpoint (Union[Unset, None, str]):
        protocols (Union[Unset, None, List[EventProtocols]]):
        statuses (Union[Unset, None, List[FsEventStatus]]):
        instance_ids (Union[Unset, None, List[str]]):
        exclude_ids (Union[Unset, None, List[str]]):
        limit (Union[Unset, None, int]):  Default: 100.
        order (Union[Unset, None, GetFsEventsOrder]):  Example: DESC.

    Returns:
        Response[Union[Any, List[FsEvent]]]
    """

    return sync_detailed(
        client=client,
        start_timestamp=start_timestamp,
        end_timestamp=end_timestamp,
        actions=actions,
        username=username,
        ip=ip,
        ssh_cmd=ssh_cmd,
        fs_provider=fs_provider,
        bucket=bucket,
        endpoint=endpoint,
        protocols=protocols,
        statuses=statuses,
        instance_ids=instance_ids,
        exclude_ids=exclude_ids,
        limit=limit,
        order=order,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    start_timestamp: Union[Unset, None, int] = 0,
    end_timestamp: Union[Unset, None, int] = 0,
    actions: Union[Unset, None, List[FsEventAction]] = UNSET,
    username: Union[Unset, None, str] = UNSET,
    ip: Union[Unset, None, str] = UNSET,
    ssh_cmd: Union[Unset, None, str] = UNSET,
    fs_provider: Union[Unset, None, FsProviders] = UNSET,
    bucket: Union[Unset, None, str] = UNSET,
    endpoint: Union[Unset, None, str] = UNSET,
    protocols: Union[Unset, None, List[EventProtocols]] = UNSET,
    statuses: Union[Unset, None, List[FsEventStatus]] = UNSET,
    instance_ids: Union[Unset, None, List[str]] = UNSET,
    exclude_ids: Union[Unset, None, List[str]] = UNSET,
    limit: Union[Unset, None, int] = 100,
    order: Union[Unset, None, GetFsEventsOrder] = UNSET,
) -> Response[Union[Any, List[FsEvent]]]:
    """Get filesystem events

     Returns an array with one or more filesystem events applying the specified filters. This API is only
    available if you configure an \"eventsearcher\" plugin

    Args:
        start_timestamp (Union[Unset, None, int]):
        end_timestamp (Union[Unset, None, int]):
        actions (Union[Unset, None, List[FsEventAction]]):
        username (Union[Unset, None, str]):
        ip (Union[Unset, None, str]):
        ssh_cmd (Union[Unset, None, str]):
        fs_provider (Union[Unset, None, FsProviders]): Filesystem providers:
              * `0` - Local filesystem
              * `1` - S3 Compatible Object Storage
              * `2` - Google Cloud Storage
              * `3` - Azure Blob Storage
              * `4` - Local filesystem encrypted
              * `5` - SFTP
        bucket (Union[Unset, None, str]):
        endpoint (Union[Unset, None, str]):
        protocols (Union[Unset, None, List[EventProtocols]]):
        statuses (Union[Unset, None, List[FsEventStatus]]):
        instance_ids (Union[Unset, None, List[str]]):
        exclude_ids (Union[Unset, None, List[str]]):
        limit (Union[Unset, None, int]):  Default: 100.
        order (Union[Unset, None, GetFsEventsOrder]):  Example: DESC.

    Returns:
        Response[Union[Any, List[FsEvent]]]
    """

    kwargs = _get_kwargs(
        client=client,
        start_timestamp=start_timestamp,
        end_timestamp=end_timestamp,
        actions=actions,
        username=username,
        ip=ip,
        ssh_cmd=ssh_cmd,
        fs_provider=fs_provider,
        bucket=bucket,
        endpoint=endpoint,
        protocols=protocols,
        statuses=statuses,
        instance_ids=instance_ids,
        exclude_ids=exclude_ids,
        limit=limit,
        order=order,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.request(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    start_timestamp: Union[Unset, None, int] = 0,
    end_timestamp: Union[Unset, None, int] = 0,
    actions: Union[Unset, None, List[FsEventAction]] = UNSET,
    username: Union[Unset, None, str] = UNSET,
    ip: Union[Unset, None, str] = UNSET,
    ssh_cmd: Union[Unset, None, str] = UNSET,
    fs_provider: Union[Unset, None, FsProviders] = UNSET,
    bucket: Union[Unset, None, str] = UNSET,
    endpoint: Union[Unset, None, str] = UNSET,
    protocols: Union[Unset, None, List[EventProtocols]] = UNSET,
    statuses: Union[Unset, None, List[FsEventStatus]] = UNSET,
    instance_ids: Union[Unset, None, List[str]] = UNSET,
    exclude_ids: Union[Unset, None, List[str]] = UNSET,
    limit: Union[Unset, None, int] = 100,
    order: Union[Unset, None, GetFsEventsOrder] = UNSET,
) -> Optional[Union[Any, List[FsEvent]]]:
    """Get filesystem events

     Returns an array with one or more filesystem events applying the specified filters. This API is only
    available if you configure an \"eventsearcher\" plugin

    Args:
        start_timestamp (Union[Unset, None, int]):
        end_timestamp (Union[Unset, None, int]):
        actions (Union[Unset, None, List[FsEventAction]]):
        username (Union[Unset, None, str]):
        ip (Union[Unset, None, str]):
        ssh_cmd (Union[Unset, None, str]):
        fs_provider (Union[Unset, None, FsProviders]): Filesystem providers:
              * `0` - Local filesystem
              * `1` - S3 Compatible Object Storage
              * `2` - Google Cloud Storage
              * `3` - Azure Blob Storage
              * `4` - Local filesystem encrypted
              * `5` - SFTP
        bucket (Union[Unset, None, str]):
        endpoint (Union[Unset, None, str]):
        protocols (Union[Unset, None, List[EventProtocols]]):
        statuses (Union[Unset, None, List[FsEventStatus]]):
        instance_ids (Union[Unset, None, List[str]]):
        exclude_ids (Union[Unset, None, List[str]]):
        limit (Union[Unset, None, int]):  Default: 100.
        order (Union[Unset, None, GetFsEventsOrder]):  Example: DESC.

    Returns:
        Response[Union[Any, List[FsEvent]]]
    """

    return (
        await asyncio_detailed(
            client=client,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            actions=actions,
            username=username,
            ip=ip,
            ssh_cmd=ssh_cmd,
            fs_provider=fs_provider,
            bucket=bucket,
            endpoint=endpoint,
            protocols=protocols,
            statuses=statuses,
            instance_ids=instance_ids,
            exclude_ids=exclude_ids,
            limit=limit,
            order=order,
        )
    ).parsed



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.secret import Secret
from ..types import UNSET, Unset

T = TypeVar("T", bound="RecoveryCode")


@attr.s(auto_attribs=True)
class RecoveryCode:
    """Recovery codes to use if the user loses access to their second factor auth device. Each code can only be used once,
    you should use these codes to login and disable or reset 2FA for your account

        Attributes:
            secret (Union[Unset, Secret]): The secret is encrypted before saving, so to set a new secret you must provide a
                payload and set the status to "Plain". The encryption key and additional data will be generated automatically.
                If you set the status to "Redacted" the existig secret will be preserved
            used (Union[Unset, bool]):
    """

    secret: Union[Unset, Secret] = UNSET
    used: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        secret: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.secret, Unset):
            secret = self.secret.to_dict()

        used = self.used

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if secret is not UNSET:
            field_dict["secret"] = secret
        if used is not UNSET:
            field_dict["used"] = used

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _secret = d.pop("secret", UNSET)
        secret: Union[Unset, Secret]
        if isinstance(_secret, Unset):
            secret = UNSET
        else:
            secret = Secret.from_dict(_secret)

        used = d.pop("used", UNSET)

        recovery_code = cls(
            secret=secret,
            used=used,
        )

        recovery_code.additional_properties = d
        return recovery_code

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.base_totp_config import BaseTOTPConfig
from ..models.recovery_code import RecoveryCode
from ..types import UNSET, Unset

T = TypeVar("T", bound="AdminFilters")


@attr.s(auto_attribs=True)
class AdminFilters:
    """
    Attributes:
        allow_list (Union[Unset, List[str]]): only clients connecting from these IP/Mask are allowed. IP/Mask must be in
            CIDR notation as defined in RFC 4632 and RFC 4291, for example "192.0.2.0/24" or "2001:db8::/32" Example:
            ['192.0.2.0/24', '2001:db8::/32'].
        allow_api_key_auth (Union[Unset, bool]): API key auth allows to impersonate this administrator with an API key
        totp_config (Union[Unset, BaseTOTPConfig]):
        recovery_codes (Union[Unset, List[RecoveryCode]]):
    """

    allow_list: Union[Unset, List[str]] = UNSET
    allow_api_key_auth: Union[Unset, bool] = UNSET
    totp_config: Union[Unset, BaseTOTPConfig] = UNSET
    recovery_codes: Union[Unset, List[RecoveryCode]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allow_list: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allow_list, Unset):
            allow_list = self.allow_list

        allow_api_key_auth = self.allow_api_key_auth
        totp_config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.totp_config, Unset):
            totp_config = self.totp_config.to_dict()

        recovery_codes: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.recovery_codes, Unset):
            recovery_codes = []
            for recovery_codes_item_data in self.recovery_codes:
                recovery_codes_item = recovery_codes_item_data.to_dict()

                recovery_codes.append(recovery_codes_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allow_list is not UNSET:
            field_dict["allow_list"] = allow_list
        if allow_api_key_auth is not UNSET:
            field_dict["allow_api_key_auth"] = allow_api_key_auth
        if totp_config is not UNSET:
            field_dict["totp_config"] = totp_config
        if recovery_codes is not UNSET:
            field_dict["recovery_codes"] = recovery_codes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allow_list = cast(List[str], d.pop("allow_list", UNSET))

        allow_api_key_auth = d.pop("allow_api_key_auth", UNSET)

        _totp_config = d.pop("totp_config", UNSET)
        totp_config: Union[Unset, BaseTOTPConfig]
        if isinstance(_totp_config, Unset):
            totp_config = UNSET
        else:
            totp_config = BaseTOTPConfig.from_dict(_totp_config)

        recovery_codes = []
        _recovery_codes = d.pop("recovery_codes", UNSET)
        for recovery_codes_item_data in _recovery_codes or []:
            recovery_codes_item = RecoveryCode.from_dict(recovery_codes_item_data)

            recovery_codes.append(recovery_codes_item)

        admin_filters = cls(
            allow_list=allow_list,
            allow_api_key_auth=allow_api_key_auth,
            totp_config=totp_config,
            recovery_codes=recovery_codes,
        )

        admin_filters.additional_properties = d
        return admin_filters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
